local addonName, ns = ...
local _G = _G
local addon = ns.addon

local xpBarTemplate = addon.xpBarTemplate
addon.repBarTemplate = addon:newTemplate(xpBarTemplate)
local repBarTemplate = addon.repBarTemplate
local setfenv = _G.setfenv
setfenv(1, repBarTemplate)

local type = _G.type
local assert = _G.assert
local min, max  = _G.min, _G.max

local REP_DATA, REP_NAMES = ns.REP_DATA, ns.REP_NAMES

local oTA = repBarTemplate.optionsTable.args.range.args
oTA.startXP.name = "Start Reputation"
oTA.startXP.desc = "Minimum reputation to display."
oTA.targetXP.name = "Target Reputation"
oTA.targetXP.desc = "Maximum reputation to display."

-- start xp
function repBarTemplate:getStartXPMin()
    return -42000
end

-- target level
function repBarTemplate:getTargetLevelMax()
    local currentLevel = self:getCurrentLevel()
    local dataMax =  #REP_DATA
    return max(currentLevel,dataMax)
end

function repBarTemplate:getTargetXPMax()
    local dataMax = self:calcXpForLevel(#REP_DATA)
    local currentXP = self:getCurrentXP()
    return max(dataMax,currentXP)
end

function repBarTemplate:calcXpForLevel(level)
    --assert(level <= 8,"level=" .. tostring(level))
    if level > 8 then
        return self:calcXpForLevel(8)
    end
    local sum = repBarTemplate:getStartXPMin()
    for i=1,level-1 do
        sum = sum + REP_DATA[i]
    end
    return sum
end

function repBarTemplate:calcLevelForXP(xp)
    local level = 1
    while xp > self:calcXpForLevel(level+1) and level <= 8 do
        level = level + 1
    end
    local startXP = self:calcXpForLevel(level)
    assert(startXP <= xp,"level=" .. level .. " ,xp=" .. xp .. " ,startXP=" .. startXP)
    if startXP == xp then
        return level
    end
    local nextXP = self:calcXpForLevel(level+1)
    local fraction
    if nextXP > startXP then
        fraction = (xp-startXP)/(nextXP-startXP)
    else
        fraction = 1
    end
    assert(0.0 <= fraction and fraction <= 1.0, "fraction=" .. fraction .. ", level=" .. level .. " ,xp=" .. xp .. " ,startXP=" .. startXP)
    return level + fraction
end

function repBarTemplate:drawReputationNumbers()
    local barCache = self.cache
    local frame = barCache.frame
    local repOptions = self:getOptions()
    local startGlobalXP = repOptions.startXP or self:getCurrentXP()
    local targetGlobalXP = repOptions.targetXP or self:getTargetXPMax()
    if not startGlobalXP or not targetGlobalXP then
        return
    end
    local width = frame:GetRight() - frame:GetLeft()
    -- textures and numbers for levels
    barCache.levelTextures = barCache.levelTextures or {}
    barCache.levelStrings = barCache.levelStrings or {}
    local levelTextures = barCache.levelTextures
    local levelStrings = barCache.levelStrings
    for level = 1,#REP_DATA do
        local xpForLevel = self:calcXpForLevel(level)
        assert(type(xpForLevel) == 'number')
        if startGlobalXP <= xpForLevel and xpForLevel <= targetGlobalXP then
            local levelGlobalXPRatio = (xpForLevel-startGlobalXP)/(targetGlobalXP-startGlobalXP)
            -- textures
            levelTextures[level] = levelTextures[level] or frame:CreateTexture(frame:GetName()..'Level'..level,'OVERLAY')
            local levelTexture = levelTextures[level]
            levelTexture:ClearAllPoints()
            levelTexture:SetPoint('TOP')
            levelTexture:SetPoint('BOTTOM')
            if levelGlobalXPRatio < 1.0 then
                levelTexture:SetPoint('LEFT',frame,'LEFT',levelGlobalXPRatio * width,0)
            else
                levelTexture:SetPoint('RIGHT')
            end
            levelTexture:SetWidth(1)
            levelTexture:SetTexture(1,1,1,1)
            levelTexture:Show()
            -- numbers
            levelStrings[level] = levelStrings[level] or frame:CreateFontString(nil,'OVERLAY')
            local levelString = levelStrings[level]
            local size = repOptions.size or self.defaults.size
            local isValid = levelString:SetFont('Fonts\\FRIZQT__.TTF',size)
            assert(isValid)
            levelString:ClearAllPoints()
            if levelGlobalXPRatio > 0 then
                levelString:SetPoint('RIGHT',levelTexture,'LEFT')
                levelString:SetJustifyH('RIGHT')
            else
                levelString:SetPoint('LEFT',levelTexture,'RIGHT')
                levelString:SetJustifyH('LEFT')
            end
            levelString:SetJustifyV('MIDDLE')
            levelString:SetText(REP_NAMES[level])
            levelString:Show()
        else
            if levelTextures[level] then
                levelTextures[level]:Hide()
            end
            if levelStrings[level] then
                levelStrings[level]:Hide()
            end
        end
    end
end