local addonName, ns = ...
local _G = _G
local MODULE_NAME = 'GuildRep'
local addon = ns.addon
local module = addon:createModule(MODULE_NAME, addon.repBarTemplate, 'AceEvent-3.0')
local setfenv = _G.setfenv
setfenv(1, module)

local assert = _G.assert
local min = _G.min

local GetGuildLevel = _G.GetGuildLevel
local GetGuildFactionInfo = _G.GetGuildFactionInfo

local REP_DATA = ns.REP_DATA
local REP_NAMES = ns.REP_NAMES

-- start xp
function module:getStartXPMin()
    return 0
end

function module:getStartLevelMin()
    return 4
end

module.oldEnable = module.OnEnable
function module:OnEnable()
    self:oldEnable()
    self:RegisterEvent('GUILD_REP_UPDATED', 'Invalidate')
    -- TODO find proper events
end

function module:Redraw()
    --addon:Debug("Guild XP Redraw")

    local xpOptions = self:getOptions()
    local barCache = self.cache

    local guildLevel = GetGuildLevel('player')
    local currentGlobalXP = self:getCurrentXP()
    -- update max settings if necessary
    if xpOptions.targetXP and currentGlobalXP > xpOptions.targetXP then
        xpOptions.targetXP = currentGlobalXP
    end
    local startGlobalXP = xpOptions.startXP
    local targetGlobalXP = xpOptions.targetXP
    local remainingGlobalXP = self:getRemainingWeeklyRep()
    local currentGlobalXPRatio =   (currentGlobalXP  -startGlobalXP)/(targetGlobalXP-startGlobalXP)
    local remainingGlobalXPRatio = (remainingGlobalXP              )/(targetGlobalXP-startGlobalXP)
    -- create textures
    assert(barCache and barCache.frame)
    local frame = barCache.frame
    local right = frame:GetRight()
    local left = frame:GetLeft()
    if not (right and left) then
        self:Invalidate()
        return
    end
    local width = frame:GetRight() - frame:GetLeft()
    -- texture for curent xp
    self:DrawPrimaryBar(currentGlobalXPRatio)

    -- texture for remaining xp
    local remainingRatio = min(remainingGlobalXPRatio,1-currentGlobalXPRatio)
    self:DrawSecondaryBar(remainingRatio)

    -- textures and numbers for levels
    self:drawReputationNumbers()

    --update options table
    self:updateOptionsTable()
end

function module:getGuildRepData()
    local _, _, level, _, maxLocal, currentLocal, _,_,_,_,_,_,_, remaining, weekMax = GetGuildFactionInfo()
    level = level-1
    assert(0 <= level and level <= #REP_DATA,"level="..level)
    return level, currentLocal, maxLocal, remaining, weekMax
end

function module:getCurrentXP()
    local level, currentLocal, _, _, _ =  self:getGuildRepData()
    return currentLocal
end

function module:getRemainingWeeklyRep()
    local _, _, _, remaining, _ = self:getGuildRepData()
    return remaining
end

function module:fillTooltip(tooltip)
    local currentGlobalXP = self:getCurrentXP()
    local targetGlobalXP = self:getOptions().targetXP or self:getTargetXPMax()
    local startGlobalXP = self:getOptions().startXP
    local remainingGlobalXP = self:getRemainingWeeklyRep()
    local deltaXP = targetGlobalXP-startGlobalXP
    self:addNameToTooltip(tooltip)
    self:fillTooltipWithSecond(tooltip, currentGlobalXP-startGlobalXP, deltaXP, remainingGlobalXP)
end

module.defaults.targetXP = module:calcXpForLevel(module:getTargetLevelMax())